from paom.base import BaseResource


class Products(BaseResource):

    ABSOLUTE_URL = "/products/"
    REQUEST_TYPES = ["GET", "POST", "ALL", "PATCH", "DELETE"]

    def collections(self, slug):
        url = "%s%s" % ("/collections-in-product/", slug)
        return  self.request._call_api(url)


class Collections(BaseResource):

    ABSOLUTE_URL = "/collections/"
    REQUEST_TYPES = ["GET", "POST", "ALL", "PATCH", "DELETE"]

    def products(self, slug):
        url = "%s%s" % ("/products-in-collection/", slug)
        return  self.request._call_api(url)

    def remove_product(self, data):
        """
        This method will remove a product from a collection. 'data' parameter
        should contain:
            'product': serialized product
            'paom_collection': serialized collection
        """
        url = "/product-collections-delete/"
        return  self.request._call_api(url, data=data,  method="DELETE")


class Designs(BaseResource):

    ABSOLUTE_URL = "/designs/"
    REQUEST_TYPES = ["GET", "POST", "ALL", "PATCH", "DELETE"]


class Categories(BaseResource):

    def products(self, slug):
        url = "%s%s" % ("/category/", slug)
        return  self.request._call_api(url)



class BaseResource(object):

    REQUEST_TYPES = []

    def __init__(self, request):
        self.request = request

    def _get_lookup(self, data):
        lookup = "id"
        if "username" in data:
            lookup = "username"
        elif "slug" in data:
            lookup = "slug"
        return lookup

    def all(self):
        if "ALL" in self.REQUEST_TYPES:
            url = self.ABSOLUTE_URL
            return self.request._call_api(url)
        else:
            raise AttributeError("Attribute not allowed for this resource.")

    def create(self, data):
        if "POST" in self.REQUEST_TYPES:
            url = self.ABSOLUTE_URL
            return self.request._call_api(url, data, method="POST")
        else:
            raise AttributeError("Attribute not allowed for this resource.")

    def update(self, data):
        if "PATCH" in self.REQUEST_TYPES:
            lookup = self._get_lookup(data)
            url = "%s%s" % (self.ABSOLUTE_URL, lookup)
            return self.request._call_api(url, data, method="PATCH")
        else:
            raise AttributeError("Attribute not allowed for this resource.")

    def delete(self, data):
        if "DELETE" in self.REQUEST_TYPES:
            lookup = self._get_lookup(data)
            url = "%s%s" % (self.ABSOLUTE_URL, lookup)
            return self.request._call_api(url, data, method="DELETE")
        else:
            raise AttributeError("Attribute not allowed for this resource.")

    def get(self, lookup):
        """
        param 'lookup' should either be 'slug' or 'id' of the
        object being retrieved.
        """
        if "GET" in self.REQUEST_TYPES:
            url = "%s%s" % (self.ABSOLUTE_URL, lookup)
            return self.request._call_api(url)
        else:
            raise AttributeError("Attribute not allowed for this resource.")



from paom.base import BaseResource


class Quizzes(BaseResource):

    ABSOLUTE_URL = "/quizzes/"
    REQUEST_TYPES = ["GET", "POST", "ALL"]


class Questions(BaseResource):

    ABSOLUTE_URL = "/questions/"
    REQUEST_TYPES = ["POST", "ALL"]


class Answers(BaseResource):

    ABSOLUTE_URL = "/answers/"
    REQUEST_TYPES = ["POST", "ALL"]


class QuizResults(BaseResource):

    ABSOLUTE_URL = "/quiz-results/"
    REQUEST_TYPES = ["POST", "ALL"]


class UserAnswers(BaseResource):

    ABSOLUTE_URL = "/user-answers/"
    REQUEST_TYPES = ["POST", "ALL"]


class UserResults(BaseResource):

    ABSOLUTE_URL = "/user-results/"
    REQUEST_TYPES = ["POST", "ALL"]



from paom.base import BaseResource


class Orders(BaseResource):

    ABSOLUTE_URL = "/orders/"
    REQUEST_TYPES = ["GET", "POST", "ALL", "PATCH", "DELETE"]


class Carts(BaseResource):

    ABSOLUTE_URL = "/cart/"


class CartItems(BaseResource):

    ABSOLUTE_URL = "/cart-items/"
    REQUEST_TYPES = ["GET", "POST", "PATCH", "DELETE"]



from paom.base import BaseResource


class Users(BaseResource):

    ABSOLUTE_URL = "/users/"

    def favorites(self, username):
        url = "%s%s" % ("/user-favorites/", username)
        return  self.request._call_api(url)

    def follow(self, data):
        """
        Allows a user to follow another user. 'data' param should
        contain:
            'follower': serialized user who is the follower
            'followed': serialized user who is being followed
        """
        url = "/user-follows/"
        return  self.request._call_api(url, data=data, method="POST")

    def unfollow(self, data):
        """
        Allows a user to unfollow another user. 'data' param should
        contain:
            'follower': serialized user who is the follower
            'followed': serialized user who is being followed
        """
        url = "/user-unfollow/"
        return  self.request._call_api(url, data=data, method="DELETE")

    def favorite_product(self, data):
        """
        Allows a user to favorite a product. 'data' param should
        contain:
            'paomuser': serialized user
            'product': serialized product
        """
        url = "/user-favorites/"
        return  self.request._call_api(url, data=data, method="POST")

    def unfavorite_product(self, data):
        """
        Allows a user to favorite a product. 'data' param should
        contain:
            'paomuser': serialized user
            'product': serialized product
        """
        url = "/user-unfavorite/"
        return  self.request._call_api(url, data=data, method="DELETE")




from setuptools import find_packages, setup
setup(
    name='paom',
    include_package_data=True,
    package_data={
        'resources': [
            '__init__.py',
            'orders.py',
            'products.py',
            'users.py',
            'quizzes.py'
        ]
    },
    packages=find_packages(exclude=["dist"]),
    version='0.5.4',
    license='MIT',
    description='API integration of PAOM',
    author='Meredith Finkelstein Chang',
    author_email='meredith@printallover.me',
    url='https://bitbucket.org/msrobot0/paom_python_api', # use the URL to the github repo
    download_url='https://bitbucket.org/msrobot0/paom_python_api/tarball/0.3',
    keywords=['paom', 'python', 'paom api'], # arbitrary keywords
    classifiers=[],
    install_requires=["requests"],
)
